package de.mlessmann.certassist.web;

import de.mlessmann.certassist.models.Certificate;
import de.mlessmann.certassist.models.CertificateInfo;
import de.mlessmann.certassist.models.CertificateInfoSubject;
import de.mlessmann.certassist.repositories.CertificateRepository;
import de.mlessmann.certassist.service.CertificateCreationService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import lombok.RequiredArgsConstructor;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/api")
@RequiredArgsConstructor(onConstructor_ = @Autowired)
public class CertificatesEndpoint {

    private final CertificateRepository certificateRepository;
    private final CertificateCreationService certificateCreationService;

    @GetMapping("/certificates")
    @Operation(description = "Fetches certificates", responses = {
            @ApiResponse(responseCode = "200", description = "Contains the returned certificates of the requested page.")
    })
    public ResponseEntity<DocumentedPage<Certificate>> getCertificates(@ParameterObject Pageable pageable) {
        var certificates = certificateRepository.findAll(pageable);
        return ResponseEntity.ok(DocumentedPage.of(certificates));
    }

    @PostMapping("/certificates")
    @Operation(description = "Requests a new certificate", responses = {
            @ApiResponse(responseCode = "400", description = "One of the provided parameters is invalid."),
            @ApiResponse(responseCode = "200", description = "Returns the newly created certificate.")
    })
    public ResponseEntity<Certificate> createCertificate(@RequestBody CertificateInfo request) {
        var createdCertificate = certificateCreationService.createCertificate(
                CertificateInfo.builder()
                        .type(CertificateInfo.RequestType.STANDALONE_CERTIFICATE)
                        .issuer(CertificateInfoSubject.builder()
                                .commonName("Test")
                        )
                        .build()
        );
        return ResponseEntity.ok(createdCertificate);
    }

}
